@@ -1,4 +1,4 @@
-/* A simple standalone XML-RPC server based on Abyss that processes a
+/* A simple standalone XML-RPC server program based on Abyss that processes a
    single RPC from an existing TCP connection on Standard Input.
 
    A typical example of where this would be useful is with an Inetd
@@ -21,11 +21,14 @@
    respond to the client, then exit.
 */
 
+#define _XOPEN_SOURCE 600
+#define WIN32_LEAN_AND_MEAN  /* required by xmlrpc-c/server_abyss.h */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <signal.h>
-#ifndef WIN32
-#include <unistd.h>
+#ifndef _WIN32
+#  include <unistd.h>
 #endif
 
 #include <xmlrpc-c/base.h>
@@ -48,7 +51,7 @@ setupSignalHandlers(void) {
        obviously don't want to die just because a client didn't complete
        an RPC, so we ignore SIGPIPE.
     */
-#ifndef WIN32
+#ifndef _WIN32
     struct sigaction mysigaction;
     
     sigemptyset(&mysigaction.sa_mask);
@@ -63,8 +66,8 @@ setupSignalHandlers(void) {
 static xmlrpc_value *
 sample_add(xmlrpc_env *   const envP, 
            xmlrpc_value * const paramArrayP,
-           void *         const serverInfo ATTR_UNUSED,
-           void *         const channelInfo ATTR_UNUSED) {
+           void *         const serverInfo,
+           void *         const channelInfo) {
     
     xmlrpc_int x, y, z;
 
