@@ -11,7 +11,7 @@
 #define  XMLRPC_CLIENT_H_INCLUDED
 
 #include <stdarg.h>
-#include <xmlrpc-c/c_util.h>
+#include <xmlrpc-c/c_util.h>  /* For XMLRPC_DLLEXPORT */
 #include <xmlrpc-c/base.h>
 
 #ifdef __cplusplus
@@ -123,6 +123,10 @@ struct xmlrpc_curl_xportparms {
     const char * proxy_userpwd;
     xmlrpc_bool  gssapi_delegation;
     const char * referer;
+    unsigned int connect_timeout;
+    xmlrpc_bool  tcp_keepalive;
+    unsigned int tcp_keepidle_sec;
+    unsigned int tcp_keepintvl_sec;
 };
 
 
@@ -173,13 +177,13 @@ struct xmlrpc_clientparms {
 
 /* XMLRPC_CPSIZE(xyz) is the minimum size a struct xmlrpc_clientparms
    must be to include the 'xyz' member.  This is essential to forward and
-   backward compatbility, as new members will be added to the end of the
+   backward compatibility, as new members will be added to the end of the
    struct in future releases.  This is how the callee knows whether or
    not the caller is new enough to have supplied a certain parameter.
 */
 
 XMLRPC_CLIENT_EXPORTED
-const char * 
+const char *
 xmlrpc_client_get_default_transport(xmlrpc_env * const env);
 
 /* A user's function to handle the response to an asynchronous call.
@@ -221,7 +225,7 @@ xmlrpc_server_info_new(xmlrpc_env * const envP,
 
 /* Create a new server info record, with a copy of the old server. */
 XMLRPC_CLIENT_EXPORTED
-extern xmlrpc_server_info * 
+extern xmlrpc_server_info *
 xmlrpc_server_info_copy(xmlrpc_env *         const envP,
                         xmlrpc_server_info * const srcP);
 
@@ -231,14 +235,14 @@ xmlrpc_server_info_free(xmlrpc_server_info * const serverP);
 
 
 XMLRPC_CLIENT_EXPORTED
-void 
+void
 xmlrpc_server_info_set_user(xmlrpc_env *         const envP,
                             xmlrpc_server_info * const serverInfoP,
                             const char *         const username,
                             const char *         const password);
 
 XMLRPC_CLIENT_EXPORTED
-void 
+void
 xmlrpc_server_info_set_basic_auth(xmlrpc_env *         const envP,
                                   xmlrpc_server_info * const serverP,
                                   const char *         const username,
@@ -306,7 +310,7 @@ void
 xmlrpc_client_teardown_global_const(void);
 
 XMLRPC_CLIENT_EXPORTED
-void 
+void
 xmlrpc_client_create(xmlrpc_env *                      const envP,
                      int                               const flags,
                      const char *                      const appname,
@@ -316,7 +320,7 @@ xmlrpc_client_create(xmlrpc_env *                      const envP,
                      xmlrpc_client **                  const clientPP);
 
 XMLRPC_CLIENT_EXPORTED
-void 
+void
 xmlrpc_client_destroy(xmlrpc_client * const clientP);
 
 XMLRPC_CLIENT_EXPORTED
@@ -358,11 +362,11 @@ xmlrpc_client_call2f_va(xmlrpc_env *               const envP,
                         va_list                          args);
 
 XMLRPC_CLIENT_EXPORTED
-void 
+void
 xmlrpc_client_event_loop_finish(xmlrpc_client * const clientP);
 
 XMLRPC_CLIENT_EXPORTED
-void 
+void
 xmlrpc_client_event_loop_finish_timeout(xmlrpc_client * const clientP,
                                         unsigned long   const milliseconds);
 
@@ -377,7 +381,7 @@ xmlrpc_client_start_rpc(xmlrpc_env *               const envP,
                         void *                     const userData);
 
 XMLRPC_CLIENT_EXPORTED
-void 
+void
 xmlrpc_client_start_rpcf(xmlrpc_env *    const envP,
                          xmlrpc_client * const clientP,
                          const char *    const serverUrl,
@@ -416,8 +420,8 @@ xmlrpc_client_set_interrupt(xmlrpc_client * const clientP,
 **    notice, this list of conditions and the following disclaimer in the
 **    documentation and/or other materials provided with the distribution.
 ** 3. The name of the author may not be used to endorse or promote products
-**    derived from this software without specific prior written permission. 
-**  
+**    derived from this software without specific prior written permission.
+**
 ** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 ** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
