@@ -9,6 +9,9 @@ typedef unsigned __int64  xmlrpc_uint64_t;
 
 #else
 #include <inttypes.h>
+#ifdef __INTERIX
+#  include <stdint.h>
+#endif
 
 typedef uint16_t xmlrpc_uint16_t;
 typedef uint32_t xmlrpc_uint32_t;
