@@ -1,11 +1,15 @@
 #ifndef LOCK_H_INCLUDED
 #define LOCK_H_INCLUDED
 
-typedef struct lock lock;
+struct lock;
 
-typedef void lockAcquireFn(lock *);
-typedef void lockReleaseFn(lock *);
-typedef void lockDestroyFn(lock *);
+/* GCC 2.95.3 <iostream> defines a function called 'lock', so we cannot
+   typedef struct lock to 'lock' here.
+*/
+
+typedef void lockAcquireFn(struct lock *);
+typedef void lockReleaseFn(struct lock *);
+typedef void lockDestroyFn(struct lock *);
 
 struct lock {
     /* To finish the job of making an abstract lock class that can use locks
