@@ -1,6 +1,7 @@
 #ifndef LOCK_WINDOWS_H_INCLUDED
 #define LOCK_WINDOWS_H_INCLUDED
 
+#include "xmlrpc-c/c_util.h"  /* For XMLRPC_DLLEXPORT */
 #include "lock.h"
 
 /*
@@ -17,7 +18,7 @@
 #endif
 
 XMLRPC_UTIL_EXPORTED
-lock *
+struct lock *
 xmlrpc_lock_create_windows(void);
 
 #endif
