@@ -6,7 +6,7 @@
 #include <string.h>
 
 #include "xmlrpc_config.h"
-#include "c_util.h"
+#include "xmlrpc-c/c_util.h"
 #include "bool.h"
 
 #ifdef __cplusplus
@@ -122,7 +122,13 @@ xmlrpc_makePrintableChar(char const input);
 #define STRSCMP(A,B) \
 	(strncmp((A), (B), sizeof(A)))
 #define STRSCAT(A,B) \
-    (strncat((A), (B), sizeof(A)-strlen(A)), *((A)+sizeof(A)-1) = '\0')
+    (strncat((A), (B), sizeof(A)-strlen(A)-1))
+
+#define MEMSSET(a,b) (memset(a, b, sizeof(*a)))
+
+#define MEMSCPY(a,b) (memcpy(a, b, sizeof(*a)))
+
+#define MEMSZERO(a) (MEMSSET(a, 0))
 
 /* We could do this, but it works only in GNU C 
 #define SSPRINTF(TARGET, REST...) \
