@@ -21,7 +21,7 @@
 #include <stdarg.h>
 
 #include <xmlrpc-c/config.h>  /* Defines XMLRPC_HAVE_WCHAR */
-#include <xmlrpc-c/c_util.h>  /* for XMLRPC_PRINTF_ATTR */
+#include <xmlrpc-c/c_util.h>  /* for XMLRPC_PRINTF_ATTR, _DLLEXPORT */
 
 #if XMLRPC_HAVE_WCHAR
 #include <wchar.h>
@@ -149,9 +149,9 @@ void xmlrpc_env_clean (xmlrpc_env* const env);
 ** fault_occurred flag. This function will make a private copy of 'string',
 ** so you retain responsibility for your copy. */
 XMLRPC_UTIL_EXPORTED
-void 
-xmlrpc_env_set_fault(xmlrpc_env * const env, 
-                     int          const faultCode, 
+void
+xmlrpc_env_set_fault(xmlrpc_env * const env,
+                     int          const faultCode,
                      const char * const faultDescription);
 
 /* The same as the above, but using varargs */
@@ -164,10 +164,10 @@ xmlrpc_set_fault_formatted_v(xmlrpc_env * const envP,
 
 /* The same as the above, but using a printf-style format string. */
 XMLRPC_UTIL_EXPORTED
-void 
-xmlrpc_env_set_fault_formatted(xmlrpc_env * const envP, 
+void
+xmlrpc_env_set_fault_formatted(xmlrpc_env * const envP,
                                int          const code,
-                               const char * const format, 
+                               const char * const format,
                                ...) XMLRPC_PRINTF_ATTR(3,4);
 
 /* This one infers XMLRPC_INTERNAL_ERROR and has a shorter name.
@@ -231,18 +231,13 @@ xmlrpc_faultf(xmlrpc_env * const envP,
 
 
 /*=========================================================================
-**  xmlrpc_mem_block
-**=========================================================================
-**  A resizable chunk of memory. This is mostly used internally, but it is
-**  also used by the public API in a few places.
-**  The struct fields are private!
-*/
+  xmlrpc_mem_block
+===========================================================================
+  A resizable chunk of memory. This is mostly used internally, but it is
+  also used by the public API in a few places.
+=========================================================================*/
 
-typedef struct _xmlrpc_mem_block {
-    size_t _size;
-    size_t _allocated;
-    void*  _block;
-} xmlrpc_mem_block;
+typedef struct _xmlrpc_mem_block xmlrpc_mem_block;
 
 /* Allocate a new xmlrpc_mem_block. */
 XMLRPC_UTIL_EXPORTED
@@ -252,23 +247,13 @@ xmlrpc_mem_block* xmlrpc_mem_block_new (xmlrpc_env* const env, size_t const size
 XMLRPC_UTIL_EXPORTED
 void xmlrpc_mem_block_free (xmlrpc_mem_block* const block);
 
-/* Initialize the contents of the provided xmlrpc_mem_block. */
-XMLRPC_UTIL_EXPORTED
-void xmlrpc_mem_block_init
-    (xmlrpc_env* const env, xmlrpc_mem_block* const block, size_t const size);
-
-/* Deallocate the contents of the provided xmlrpc_mem_block, but not the
-** block itself. */
-XMLRPC_UTIL_EXPORTED
-void xmlrpc_mem_block_clean (xmlrpc_mem_block* const block);
-
 /* Get the size and contents of the xmlrpc_mem_block. */
 XMLRPC_UTIL_EXPORTED
-size_t 
+size_t
 xmlrpc_mem_block_size(const xmlrpc_mem_block * const block);
 
 XMLRPC_UTIL_EXPORTED
-void * 
+void *
 xmlrpc_mem_block_contents(const xmlrpc_mem_block * const block);
 
 /* Resize an xmlrpc_mem_block, preserving as much of the contents as
@@ -286,10 +271,6 @@ void xmlrpc_mem_block_append
     xmlrpc_mem_block_new((env), sizeof(type) * (size))
 #define XMLRPC_MEMBLOCK_FREE(type,block) \
     xmlrpc_mem_block_free(block)
-#define XMLRPC_MEMBLOCK_INIT(type,env,block,size) \
-    xmlrpc_mem_block_init((env), (block), sizeof(type) * (size))
-#define XMLRPC_MEMBLOCK_CLEAN(type,block) \
-    xmlrpc_mem_block_clean(block)
 #define XMLRPC_MEMBLOCK_SIZE(type,block) \
     (xmlrpc_mem_block_size(block) / sizeof(type))
 #define XMLRPC_MEMBLOCK_CONTENTS(type,block) \
@@ -307,10 +288,6 @@ void xmlrpc_mem_block_append
     XMLRPC_MEMBLOCK_NEW(type,env,size)
 #define XMLRPC_TYPED_MEM_BLOCK_FREE(type,block) \
     XMLRPC_MEMBLOCK_FREE(type,block)
-#define XMLRPC_TYPED_MEM_BLOCK_INIT(type,env,block,size) \
-    XMLRPC_MEMBLOCK_INIT(type,env,block,size)
-#define XMLRPC_TYPED_MEM_BLOCK_CLEAN(type,block) \
-    XMLRPC_MEMBLOCK_CLEAN(type,block)
 #define XMLRPC_TYPED_MEM_BLOCK_SIZE(type,block) \
     XMLRPC_MEMBLOCK_SIZE(type,block)
 #define XMLRPC_TYPED_MEM_BLOCK_CONTENTS(type,block) \
@@ -326,7 +303,7 @@ void xmlrpc_mem_block_append
 **=======================================================================*/
 
 XMLRPC_UTIL_EXPORTED
-void 
+void
 xmlrpc_validate_utf8(xmlrpc_env * const envP,
                      const char * const utf8Data,
                      size_t       const utf8Len);
