@@ -32,6 +32,14 @@
 **
 ******************************************************************************/
 
+/* There is a complex bug in AIX that is worked around by declaring this as
+   X/OPEN 6 source code even though we don't depend up on any specific stuff
+   from that standard.  The AIX bug causes <unistd.h> to define "lseek64"
+   twice.  We tried _POSIX_SOURCE and that made the lseek problem go away but
+   caused <sys/socket.h> not to define struct sockaddr!   AIX 5.3 13.09.16.
+*/
+#define _XOPEN_SOURCE 600
+
 #include "xmlrpc_config.h"
 #include "mallocvar.h"
 
@@ -66,19 +74,15 @@
 #include "file.h"
 
 bool const win32 =
-#ifdef _WIN32
-TRUE;
+#if MSVCRT
+    true;
 #else
-FALSE;
+    false;
 #endif
 
 struct TFileFind {
-#ifdef _WIN32
-  #if MSVCRT
-      intptr_t handle;
-  #else
-      HANDLE handle;
-  #endif
+#if MSVCRT
+    intptr_t handle;
 #else
     char path[NAME_MAX+1];
     DIR * handle;
@@ -102,7 +106,7 @@ createFileImage(TFile **     const filePP,
 
     MALLOCVAR(fileP);
     if (fileP == NULL)
-        *succeededP = FALSE;
+        *succeededP = false;
     else {
         int rc;
 
@@ -112,10 +116,10 @@ createFileImage(TFile **     const filePP,
             rc = open(name, attrib);
 
         if (rc < 0)
-            *succeededP = FALSE;
+            *succeededP = false;
         else {
             fileP->fd = rc;
-            *succeededP = TRUE;
+            *succeededP = true;
         }
         if (!*succeededP)
             free(fileP);
@@ -132,7 +136,7 @@ FileOpen(TFile **     const filePP,
 
     bool succeeded;
 
-    createFileImage(filePP, name, attrib, FALSE, &succeeded);
+    createFileImage(filePP, name, attrib, false, &succeeded);
 
     return succeeded;
 }
@@ -146,7 +150,7 @@ FileOpenCreate(TFile **     const filePP,
 
     bool succeeded;
 
-    createFileImage(filePP, name, attrib, TRUE, &succeeded);
+    createFileImage(filePP, name, attrib, true, &succeeded);
 
     return succeeded;
 }
@@ -251,20 +255,6 @@ fileFindFirstWin(TFileFind *  const filefindP ATTR_UNUSED,
 #if MSVCRT
     filefindP->handle = _findfirsti64(search, fileinfo);
     *retP = filefindP->handle != -1;
-#else
-#ifdef _WIN32
-    filefindP->handle = FindFirstFile(search, &fileinfo->data);
-    *retP = filefindP->handle != INVALID_HANDLE_VALUE;
-    if (*retP) {
-        LARGE_INTEGER li;
-        li.LowPart = fileinfo->data.nFileSizeLow;
-        li.HighPart = fileinfo->data.nFileSizeHigh;
-        strcpy( fileinfo->name, fileinfo->data.cFileName );
-        fileinfo->attrib = fileinfo->data.dwFileAttributes;
-        fileinfo->size   = li.QuadPart;
-        fileinfo->time_write = fileinfo->data.ftLastWriteTime.dwLowDateTime;
-    }
-#endif
 #endif
     xmlrpc_strfree(search);
 }
@@ -284,7 +274,7 @@ fileFindFirstPosix(TFileFind *  const filefindP,
     if (filefindP->handle)
         *retP = FileFindNext(filefindP, fileinfo);
     else
-        *retP = FALSE;
+        *retP = false;
 #endif
 }
     
@@ -302,7 +292,7 @@ FileFindFirst(TFileFind ** const filefindPP,
     MALLOCVAR(filefindP);
 
     if (filefindP == NULL)
-        succeeded = FALSE;
+        succeeded = false;
     else {
         if (win32)
             fileFindFirstWin(filefindP, path, fileinfo, &succeeded);
@@ -325,21 +315,6 @@ fileFindNextWin(TFileFind * const filefindP ATTR_UNUSED,
 
 #if MSVCRT
     *retvalP = _findnexti64(filefindP->handle, fileinfo) != -1;
-#else
-#ifdef _WIN32
-    bool found;
-    found = FindNextFile(filefindP->handle, &fileinfo->data);
-    if (found) {
-        LARGE_INTEGER li;
-        li.LowPart = fileinfo->data.nFileSizeLow;
-        li.HighPart = fileinfo->data.nFileSizeHigh;
-        strcpy(fileinfo->name, fileinfo->data.cFileName);
-        fileinfo->attrib = fileinfo->data.dwFileAttributes;
-        fileinfo->size   = li.QuadPart;
-        fileinfo->time_write = fileinfo->data.ftLastWriteTime.dwLowDateTime;
-    }
-    *retvalP = found;
-#endif
 #endif
 }
 
@@ -350,7 +325,7 @@ fileFindNextPosix(TFileFind * const filefindP,
                   TFileInfo * const fileinfoP,
                   bool *      const retvalP) {
 
-#ifndef _WIN32
+#if !MSVCRT
     struct dirent * deP;
 
     deP = readdir(filefindP->handle);
@@ -374,9 +349,9 @@ fileFindNextPosix(TFileFind * const filefindP,
         fileinfoP->size       = fs.st_size;
         fileinfoP->time_write = fs.st_mtime;
         
-        *retvalP = TRUE;
+        *retvalP = true;
     } else
-        *retvalP = FALSE;
+        *retvalP = false;
 #endif
 }
 
@@ -400,12 +375,8 @@ FileFindNext(TFileFind * const filefindP,
 
 void
 FileFindClose(TFileFind * const filefindP) {
-#ifdef _WIN32
 #if MSVCRT
     _findclose(filefindP->handle);
-#else
-   FindClose(filefindP->handle);
-#endif
 #else
     closedir(filefindP->handle);
 #endif
