@@ -8,20 +8,21 @@
 
 #define _XOPEN_SOURCE 600  /* Make sure strdup() is in <string.h> */
 
+#include "xmlrpc_config.h"
+
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <errno.h>
-#ifdef _WIN32
+#if MSVCRT
   #include <io.h>
 #else
   #include <unistd.h>
 #endif
 #include <fcntl.h>
 
-#include "xmlrpc_config.h"
 #include "bool.h"
 #include "int.h"
 #include "girmath.h"
@@ -136,7 +137,7 @@ cmpfiledates(const TFileInfo ** const f1PP,
     else if (!(f1P->attrib & A_SUBDIR) && (f2P->attrib & A_SUBDIR))
         retval = 1;
     else {
-        assert((int)(f1P->time_write - f2P->time_write) == 
+        assert((int)(f1P->time_write - f2P->time_write) ==
                (f1P->time_write - f2P->time_write));
         retval = (int)(f1P->time_write - f2P->time_write);
     }
@@ -152,26 +153,26 @@ determineSortType(const char *  const query,
                   bool *        const textP,
                   const char ** const errorP) {
 
-    *ascendingP = TRUE;
+    *ascendingP = true;
     *sortP = 1;
-    *textP = FALSE;
+    *textP = false;
     *errorP = NULL;
-    
+
     if (query) {
         if (xmlrpc_streq(query, "plain"))
-            *textP = TRUE;
+            *textP = true;
         else if (xmlrpc_streq(query, "name-up")) {
             *sortP = 1;
-            *ascendingP = TRUE;
+            *ascendingP = true;
         } else if (xmlrpc_streq(query, "name-down")) {
             *sortP = 1;
-            *ascendingP = FALSE;
+            *ascendingP = false;
         } else if (xmlrpc_streq(query, "date-up")) {
             *sortP = 2;
-            *ascendingP = TRUE;
+            *ascendingP = true;
         } else if (xmlrpc_streq(query, "date-down")) {
             *sortP = 2;
-            *ascendingP = FALSE;
+            *ascendingP = false;
         } else  {
             xmlrpc_asprintf(errorP, "invalid query value '%s'", query);
         }
@@ -187,7 +188,7 @@ generateListing(TList *       const listP,
                 TPool *       const poolP,
                 const char ** const errorP,
                 uint16_t *    const responseStatusP) {
-    
+
     TFileInfo fileinfo;
     TFileFind * findhandleP;
 
@@ -224,13 +225,84 @@ generateListing(TList *       const listP,
         if (*errorP) {
             *responseStatusP = 500;
             ListFree(listP);
-        }            
+        }
         FileFindClose(findhandleP);
     }
 }
 
 
 
+static const char *
+plainTail(void) {
+/*----------------------------------------------------------------------------
+   The bottom part of the response body, in plain text format.
+-----------------------------------------------------------------------------*/
+    const char * retval;
+
+    xmlrpc_asprintf(&retval, SERVER_PLAIN_INFO);
+
+    return retval;
+}
+
+
+
+static const char *
+htmlTail(void) {
+/*----------------------------------------------------------------------------
+  The bottom part of the response body, in HTML format.
+-----------------------------------------------------------------------------*/
+    const char * retval;
+
+    xmlrpc_asprintf(
+        &retval,
+        "</PRE>"
+        SERVER_HTML_INFO
+        "</BODY></HTML>\r\n\r\n");
+
+    return retval;
+}
+
+
+
+static void
+sendDirectoryDocumentHeading(TSession *   const sessionP,
+                             const char * const uri,
+                             bool         const text) {
+
+    char z[4096];
+
+    if (text) {
+        size_t i;
+        char * p;
+
+        sprintf(z, "Index of %s\r\n", uri);
+        i = strlen(z)-2;
+        p = z + i + 2;
+
+        while (i > 0) {
+            *(p++) = '-';
+            --i;
+        }
+
+        *p = '\0';
+        strcat(z, "\r\n\r\n"
+               "Name                      Size      "
+               "Date-Time             Type\r\n"
+               "------------------------------------"
+               "--------------------------------------------\r\n");
+    } else {
+        sprintf(z, "<HTML><HEAD><TITLE>Index of %s</TITLE></HEAD><BODY>"
+                "<H1>Index of %s</H1><PRE>",
+                uri, uri);
+        strcat(z, "Name                      Size      "
+               "Date-Time             Type<HR WIDTH=100%>\r\n");
+    }
+
+    HTTPWriteBodyChunk(sessionP, z, strlen(z));
+}
+
+
+
 static void
 sendDirectoryDocument(TList *      const listP,
                       bool         const ascending,
@@ -246,36 +318,12 @@ sendDirectoryDocument(TList *      const listP,
     int16_t i;
     uint32_t k;
 
-    if (text) {
-        sprintf(z, "Index of %s" CRLF, uri);
-        i = strlen(z)-2;
-        p = z + i + 2;
-
-        while (i > 0) {
-            *(p++) = '-';
-            --i;
-        }
-
-        *p = '\0';
-        strcat(z, CRLF CRLF
-               "Name                      Size      "
-               "Date-Time             Type" CRLF
-               "------------------------------------"
-               "--------------------------------------------"CRLF);
-    } else {
-        sprintf(z, "<HTML><HEAD><TITLE>Index of %s</TITLE></HEAD><BODY>"
-                "<H1>Index of %s</H1><PRE>",
-                uri, uri);
-        strcat(z, "Name                      Size      "
-               "Date-Time             Type<HR WIDTH=100%>"CRLF);
-    }
-
-    HTTPWriteBodyChunk(sessionP, z, strlen(z));
+    sendDirectoryDocumentHeading(sessionP, uri, text);
 
     /* Sort the files */
     qsort(listP->item, listP->size, sizeof(void *),
           (TQSortProc)(sort == 1 ? cmpfilenames : cmpfiledates));
-    
+
     /* Write the listing */
     if (ascending)
         i = 0;
@@ -292,7 +340,7 @@ sendDirectoryDocument(TList *      const listP,
             ++i;
         else
             --i;
-            
+
         strcpy(z, fi->name);
 
         k = strlen(z);
@@ -311,12 +359,12 @@ sendDirectoryDocument(TList *      const listP,
             p = z1 + 24;
         } else {
             strcpy(z1, z);
-            
+
             ++k;
             p = z1 + k;
             while (k < 25)
                 z1[k++] = ' ';
-            
+
             z1[25] = '\0';
         }
 
@@ -342,9 +390,9 @@ sendDirectoryDocument(TList *      const listP,
                         u = 'G';
                 }
             }
-                
+
             sprintf(z3, "%5" PRIu64 " %c", fi->size, u);
-            
+
             if (xmlrpc_streq(fi->name, ".."))
                 z4 = "";
             else
@@ -355,22 +403,22 @@ sendDirectoryDocument(TList *      const listP,
         }
 
         if (text)
-            sprintf(z, "%s%s %s    %s   %s"CRLF, z1, p, z3, z2, z4);
+            sprintf(z, "%s%s %s    %s   %s\r\n", z1, p, z3, z2, z4);
         else
-            sprintf(z, "<A HREF=\"%s%s\">%s</A>%s %s    %s   %s"CRLF,
+            sprintf(z, "<A HREF=\"%s%s\">%s</A>%s %s    %s   %s\r\n",
                     fi->name, fi->attrib & A_SUBDIR ? "/" : "",
                     z1, p, z3, z2, z4);
 
         HTTPWriteBodyChunk(sessionP, z, strlen(z));
     }
-        
-    /* Write the tail of the file */
-    if (text)
-        strcpy(z, SERVER_PLAIN_INFO);
-    else
-        strcpy(z, "</PRE>" SERVER_HTML_INFO "</BODY></HTML>" CRLF CRLF);
-    
-    HTTPWriteBodyChunk(sessionP, z, strlen(z));
+
+    {
+        const char * const tail = text ? plainTail() : htmlTail();
+
+        HTTPWriteBodyChunk(sessionP, tail, strlen(tail));
+
+        xmlrpc_strfree(tail);
+    }
 }
 
 
@@ -389,13 +437,13 @@ notRecentlyModified(TSession * const sessionP,
         DateDecode(imsHdr, &valid, &datetime);
         if (valid) {
             if (MIN(fileModTime, sessionP->date) <= datetime)
-                retval = TRUE;
+                retval = true;
             else
-                retval = FALSE;
+                retval = false;
         } else
-            retval = FALSE;
+            retval = false;
     } else
-        retval = FALSE;
+        retval = false;
 
     return retval;
 }
@@ -415,7 +463,7 @@ addLastModifiedHeader(TSession * const sessionP,
         xmlrpc_strfree(lastModifiedValue);
     }
 }
-    
+
 
 
 static void
@@ -428,7 +476,7 @@ handleDirectory(TSession *   const sessionP,
     bool ascending;
     uint16_t sort;    /* 1=by name, 2=by date */
     const char * error;
-    
+
     determineSortType(sessionP->requestInfo.query,
                       &ascending, &sort, &text, &error);
 
@@ -455,21 +503,21 @@ handleDirectory(TSession *   const sessionP,
                 xmlrpc_strfree(error);
             } else {
                 ResponseStatus(sessionP, 200);
-                ResponseContentType(sessionP, 
+                ResponseContentType(sessionP,
                                     text ? "text/plain" : "text/html");
-            
+
                 addLastModifiedHeader(sessionP, fileModTime);
-            
+
                 ResponseChunked(sessionP);
                 ResponseWriteStart(sessionP);
-            
+
                 if (sessionP->requestInfo.method!=m_head)
                     sendDirectoryDocument(&list, ascending, sort, text,
                                           sessionP->requestInfo.uri, mimeTypeP,
                                           sessionP);
-            
-                HTTPWriteEndChunk(sessionP);
-            
+
+                ResponseWriteEnd(sessionP);
+
                 ListFree(&list);
             }
             PoolFree(&pool);
@@ -485,11 +533,11 @@ composeEntityHeader(const char ** const entityHeaderP,
                     uint64_t      const start,
                     uint64_t      const end,
                     uint64_t      const filesize) {
-                         
-    xmlrpc_asprintf(entityHeaderP, "Content-type: %s" CRLF
+
+    xmlrpc_asprintf(entityHeaderP, "Content-type: %s\r\n"
                     "Content-range: "
-                    "bytes %" PRIu64 "-%" PRIu64 "/%" PRIu64 CRLF
-                    "Content-length: %" PRIu64 CRLF CRLF,
+                    "bytes %" PRIu64 "-%" PRIu64 "/%" PRIu64 "\r\n"
+                    "Content-length: %" PRIu64 "\r\n\r\n",
                     mediatype, start, end, filesize, end-start+1);
 }
 
@@ -518,30 +566,32 @@ sendBody(TSession *      const sessionP,
     else {
         uint64_t i;
         for (i = 0; i <= sessionP->ranges.size; ++i) {
-            ConnWrite(sessionP->connP, "--", 2);
-            ConnWrite(sessionP->connP, BOUNDARY, strlen(BOUNDARY));
-            ConnWrite(sessionP->connP, CRLF, 2);
+            ConnWrite(sessionP->connP, "--", 2, CONN_EXPECT_MORE);
+            ConnWrite(sessionP->connP, BOUNDARY, strlen(BOUNDARY),
+                      CONN_EXPECT_MORE);
+            ConnWrite(sessionP->connP, "\r\n", 2, CONN_EXPECT_NOTHING);
 
             if (i < sessionP->ranges.size) {
                 uint64_t start;
                 uint64_t end;
                 bool decoded;
-                    
+
                 decoded = RangeDecode((char *)(sessionP->ranges.item[i]),
                                       filesize,
                                       &start, &end);
                 if (decoded) {
                     /* Entity header, not response header */
                     const char * entityHeader;
-                    
+
                     composeEntityHeader(&entityHeader, mediatype,
                                         start, end, filesize);
 
                     ConnWrite(sessionP->connP,
-                              entityHeader, strlen(entityHeader));
+                              entityHeader, strlen(entityHeader),
+                              CONN_EXPECT_NOTHING);
 
                     xmlrpc_strfree(entityHeader);
-                    
+
                     ConnWriteFromFile(sessionP->connP, fileP, start, end,
                                       buffer, sizeof(buffer), 0);
                 }
@@ -596,19 +646,19 @@ sendFileAsResponse(TSession *   const sessionP,
         ResponseStatus(sessionP, 206);
         break;
     }
-    
+
     if (sessionP->ranges.size == 0) {
         ResponseContentLength(sessionP, filesize);
         ResponseContentType(sessionP, mediatype);
     }
-    
+
     addLastModifiedHeader(sessionP, fileModTime);
 
     ResponseWriteStart(sessionP);
 
     if (sessionP->requestInfo.method != m_head)
         sendBody(sessionP, fileP, filesize, mediatype, start, end);
-}        
+}
 
 
 
@@ -623,7 +673,7 @@ handleFile(TSession *   const sessionP,
 -----------------------------------------------------------------------------*/
     TFile * fileP;
     bool success;
-    
+
     success = FileOpen(&fileP, fileName, O_BINARY | O_RDONLY);
     if (!success)
         ResponseStatusErrno(sessionP);
@@ -644,7 +694,7 @@ handleFile(TSession *   const sessionP,
 static void
 convertToNativeFileName(char * const fileName ATTR_UNUSED) {
 
-#ifdef _WIN32
+#if MSVCRT
     char * p;
     p = &fileName[0];
     while (*p) {
@@ -653,7 +703,7 @@ convertToNativeFileName(char * const fileName ATTR_UNUSED) {
 
         ++p;
     }
-#endif  /* _WIN32 */
+#endif
 }
 
 
@@ -668,11 +718,11 @@ HandlerDefaultBuiltin(TSession * const sessionP) {
     TFileStat fs;
     bool endingslash;
 
-    endingslash = FALSE;  /* initial value */
+    endingslash = false;  /* initial value */
 
-    if (!RequestValidURIPath(sessionP)) {
+    if (!HTTPRequestHasValidUriPath(sessionP)) {
         ResponseStatus(sessionP, 400);
-        return TRUE;
+        return true;
     }
 
     /* Must check for * (asterisk uri) in the future */
@@ -680,14 +730,14 @@ HandlerDefaultBuiltin(TSession * const sessionP) {
         ResponseAddField(sessionP, "Allow", "GET, HEAD");
         ResponseContentLength(sessionP, 0);
         ResponseStatus(sessionP, 200);
-        return TRUE;
+        return true;
     }
 
     if ((sessionP->requestInfo.method != m_get) &&
         (sessionP->requestInfo.method != m_head)) {
         ResponseAddField(sessionP, "Allow", "GET, HEAD");
         ResponseStatus(sessionP, 405);
-        return TRUE;
+        return true;
     }
 
     strcpy(z, handlerP->filesPath);
@@ -695,7 +745,7 @@ HandlerDefaultBuiltin(TSession * const sessionP) {
 
     p = z + strlen(z) - 1;
     if (*p == '/') {
-        endingslash = TRUE;
+        endingslash = true;
         *p = '\0';
     }
 
@@ -703,7 +753,7 @@ HandlerDefaultBuiltin(TSession * const sessionP) {
 
     if (!FileStat(z, &fs)) {
         ResponseStatusErrno(sessionP);
-        return TRUE;
+        return true;
     }
 
     if (fs.st_mode & S_IFDIR) {
@@ -718,7 +768,7 @@ HandlerDefaultBuiltin(TSession * const sessionP) {
             ResponseAddField(sessionP, "Location", z);
             ResponseStatus(sessionP, 302);
             ResponseWriteStart(sessionP);
-            return TRUE;
+            return true;
         }
 
         *p = DIRECTORY_SEPARATOR[0];
@@ -727,7 +777,7 @@ HandlerDefaultBuiltin(TSession * const sessionP) {
             unsigned int i;
             i = handlerP->defaultFileNames.size;
             while (i-- > 0) {
-                *p = '\0';        
+                *p = '\0';
                 strcat(z, (handlerP->defaultFileNames.item[i]));
                 if (FileStat(z, &fs)) {
                     if (!(fs.st_mode & S_IFDIR))
@@ -738,16 +788,16 @@ HandlerDefaultBuiltin(TSession * const sessionP) {
         }
 
         *(p-1) = '\0';
-        
+
         if (!FileStat(z, &fs)) {
             ResponseStatusErrno(sessionP);
-            return TRUE;
+            return true;
         }
         handleDirectory(sessionP, z, fs.st_mtime, handlerP->mimeTypeP);
     } else
         handleFile(sessionP, z, fs.st_mtime, handlerP->mimeTypeP);
 
-    return TRUE;
+    return true;
 }
 
 
@@ -774,7 +824,7 @@ size_t const HandlerDefaultBuiltinStack = 1024;
 **    documentation and/or other materials provided with the distribution.
 ** 3. The name of the author may not be used to endorse or promote products
 **    derived from this software without specific prior written permission.
-** 
+**
 ** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 ** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
