@@ -46,17 +46,18 @@ ResponseError2(TSession *   const sessionP,
     ResponseAddField(sessionP, "Content-type", "text/html");
 
     ResponseWriteStart(sessionP);
-    
+
     xmlrpc_asprintf(&errorDocument,
                     "<HTML><HEAD><TITLE>Error %d</TITLE></HEAD>"
                     "<BODY>"
                     "<H1>Error %d</H1>"
-                    "<P>%s</P>" SERVER_HTML_INFO 
+                    "<P>%s</P>" SERVER_HTML_INFO
                     "</BODY>"
                     "</HTML>",
                     sessionP->status, sessionP->status, explanation);
-    
-    ConnWrite(sessionP->connP, errorDocument, strlen(errorDocument)); 
+
+    ConnWrite(sessionP->connP, errorDocument, strlen(errorDocument),
+              CONN_EXPECT_NOTHING);
 
     xmlrpc_strfree(errorDocument);
 }
@@ -74,18 +75,21 @@ ResponseError(TSession * const sessionP) {
 
 abyss_bool
 ResponseChunked(TSession * const sessionP) {
-    /* This is only a hope, things will be real only after a call of
-       ResponseWriteStart()
-    */
+/*----------------------------------------------------------------------------
+   Make the response chunked.
+
+   This is only a hope, things will be real only after a call of
+   ResponseWriteStart()
+-----------------------------------------------------------------------------*/
     assert(!sessionP->responseStarted);
 
     sessionP->chunkedwrite =
         (sessionP->version.major > 1) ||
         (sessionP->version.major == 1 && (sessionP->version.minor >= 1));
 
-    sessionP->chunkedwritemode = TRUE;
+    sessionP->chunkedwritemode = true;
 
-    return TRUE;
+    return true;
 }
 
 
@@ -93,7 +97,9 @@ ResponseChunked(TSession * const sessionP) {
 void
 ResponseStatus(TSession *     const sessionP,
                unsigned short const code) {
-
+/*----------------------------------------------------------------------------
+   Set the status code for the response to 'code'.
+-----------------------------------------------------------------------------*/
     sessionP->status = code;
 }
 
@@ -101,7 +107,9 @@ ResponseStatus(TSession *     const sessionP,
 
 xmlrpc_uint16_t
 ResponseStatusFromErrno(int const errnoArg) {
-
+/*----------------------------------------------------------------------------
+   The appropriate HTTP status code for a POSIX errno of 'errnoArg'.
+-----------------------------------------------------------------------------*/
     uint16_t code;
 
     switch (errnoArg) {
@@ -121,7 +129,10 @@ ResponseStatusFromErrno(int const errnoArg) {
 
 void
 ResponseStatusErrno(TSession * const sessionP) {
-
+/*----------------------------------------------------------------------------
+   Set the response state from the global 'errno' value (i.e. the result
+   of the last system call Caller made).
+-----------------------------------------------------------------------------*/
     ResponseStatus(sessionP, ResponseStatusFromErrno(errno));
 }
 
@@ -165,7 +176,7 @@ ResponseAddField(TSession *   const sessionP,
                  const char * const value) {
 
     abyss_bool succeeded;
-    
+
     if (!isValidHttpToken(name)) {
         TraceMsg("Supplied HTTP header field name is not a valid HTTP token");
         succeeded = false;
@@ -187,7 +198,7 @@ addConnectionHeaderFld(TSession * const sessionP) {
 
     if (HTTPKeepalive(sessionP)) {
         const char * keepaliveValue;
-        
+
         ResponseAddField(sessionP, "Connection", "Keep-Alive");
 
         xmlrpc_asprintf(&keepaliveValue, "timeout=%u, max=%u",
@@ -199,7 +210,7 @@ addConnectionHeaderFld(TSession * const sessionP) {
     } else
         ResponseAddField(sessionP, "Connection", "close");
 }
-    
+
 
 
 static void
@@ -298,6 +309,8 @@ sendHeader(TConn * const connP,
    fields[] contains syntactically valid HTTP header field names and values.
    But to the extent that int contains undefined field names or semantically
    invalid values, the header we send is invalid.
+
+   Tell connection to expect more data for the same logical message.
 -----------------------------------------------------------------------------*/
     unsigned int i;
 
@@ -308,7 +321,7 @@ sendHeader(TConn * const connP,
         const char * line;
 
         xmlrpc_asprintf(&line, "%s: %s\r\n", fieldP->name, fieldValue);
-        ConnWrite(connP, line, strlen(line));
+        ConnWrite(connP, line, strlen(line), CONN_EXPECT_MORE);
         xmlrpc_strfree(line);
         xmlrpc_strfree(fieldValue);
     }
@@ -335,13 +348,13 @@ ResponseWriteStart(TSession * const sessionP) {
         sessionP->status = 500;
     }
 
-    sessionP->responseStarted = TRUE;
+    sessionP->responseStarted = true;
 
     {
         const char * const reason = HTTPReasonByStatus(sessionP->status);
         const char * line;
         xmlrpc_asprintf(&line,"HTTP/1.1 %u %s\r\n", sessionP->status, reason);
-        ConnWrite(sessionP->connP, line, strlen(line));
+        ConnWrite(sessionP->connP, line, strlen(line), CONN_EXPECT_MORE);
         xmlrpc_strfree(line);
     }
 
@@ -361,7 +374,7 @@ ResponseWriteStart(TSession * const sessionP) {
     */
     sendHeader(sessionP->connP, sessionP->responseHeaderFields);
 
-    ConnWrite(sessionP->connP, "\r\n", 2);  
+    ConnWrite(sessionP->connP, "\r\n", 2, CONN_EXPECT_NOTHING);
 }
 
 
@@ -398,7 +411,7 @@ ResponseContentLength(TSession *      const sessionP,
                       xmlrpc_uint64_t const len) {
 
     char contentLengthValue[32];
-    
+
     sprintf(contentLengthValue, "%" PRIu64, len);
 
     return ResponseAddField(sessionP, "Content-length", contentLengthValue);
@@ -407,7 +420,7 @@ ResponseContentLength(TSession *      const sessionP,
 
 
 void
-ResponseAccessControl(TSession *        const abyssSessionP, 
+ResponseAccessControl(TSession *        const abyssSessionP,
                       ResponseAccessCtl const accessControl) {
 
     if (accessControl.allowOrigin) {
@@ -446,7 +459,7 @@ static MIMEType * globalMimeTypeP = NULL;
 
 MIMEType *
 MIMETypeCreate(void) {
- 
+
     MIMEType * MIMETypeP;
 
     MALLOCVAR(MIMETypeP);
@@ -500,7 +513,7 @@ mimeTypeAdd(MIMEType *   const MIMETypeP,
             const char * const type,
             const char * const ext,
             bool *       const successP) {
-    
+
     uint16_t index;
     void * mimeTypesItem;
     bool typeIsInList;
@@ -518,7 +531,7 @@ mimeTypeAdd(MIMEType *   const MIMETypeP,
         extIsInList = ListFindString(&MIMETypeP->extList, ext, &index);
         if (extIsInList) {
             MIMETypeP->typeList.item[index] = mimeTypesItem;
-            *successP = TRUE;
+            *successP = true;
         } else {
             void * extItem = (void*)PoolStrdup(&MIMETypeP->pool, ext);
             if (extItem) {
@@ -528,20 +541,20 @@ mimeTypeAdd(MIMEType *   const MIMETypeP,
                     ListAdd(&MIMETypeP->typeList, mimeTypesItem);
                 if (addedToMimeTypes) {
                     bool addedToExt;
-                    
+
                     addedToExt = ListAdd(&MIMETypeP->extList, extItem);
                     *successP = addedToExt;
                     if (!*successP)
                         ListRemove(&MIMETypeP->typeList);
                 } else
-                    *successP = FALSE;
+                    *successP = false;
                 if (!*successP)
                     PoolReturn(&MIMETypeP->pool, extItem);
             } else
-                *successP = FALSE;
+                *successP = false;
         }
     } else
-        *successP = FALSE;
+        *successP = false;
 }
 
 
@@ -557,8 +570,8 @@ MIMETypeAdd2(MIMEType *   const MIMETypeArg,
     bool success;
 
     if (MIMETypeP == NULL)
-        success = FALSE;
-    else 
+        success = false;
+    else
         mimeTypeAdd(MIMETypeP, type, ext, &success);
 
     return success;
@@ -590,7 +603,7 @@ mimeTypeFromExt(MIMEType *   const MIMETypeP,
         retval = NULL;
     else
         retval = MIMETypeP->typeList.item[extindex];
-    
+
     return retval;
 }
 
@@ -632,15 +645,15 @@ findExtension(const char *  const fileName,
     unsigned int i;
 
     /* We're looking for the last dot after the last slash */
-    for (i = 0, extFound = FALSE; fileName[i]; ++i) {
+    for (i = 0, extFound = false; fileName[i]; ++i) {
         char const c = fileName[i];
-        
+
         if (c == '.') {
-            extFound = TRUE;
+            extFound = true;
             extPos = i + 1;
         }
         if (c == '/')
-            extFound = FALSE;
+            extFound = false;
     }
 
     if (extFound)
@@ -659,7 +672,7 @@ mimeTypeFromFileName(MIMEType *   const MIMETypeP,
     const char * ext;
 
     assert(MIMETypeP != NULL);
-    
+
     findExtension(fileName, &ext);
 
     if (ext)
@@ -677,7 +690,7 @@ MIMETypeFromFileName2(MIMEType *   const MIMETypeArg,
                       const char * const fileName) {
 
     const char * retval;
-    
+
     MIMEType * MIMETypeP = MIMETypeArg ? MIMETypeArg : globalMimeTypeP;
 
     if (MIMETypeP == NULL)
@@ -716,30 +729,30 @@ fileContainsText(const char * const fileName) {
         unsigned int i;
 
         readRc = FileRead(fileP, buffer, sizeof(buffer));
-       
+
         if (readRc >= 0) {
             unsigned int bytesRead = readRc;
             bool nonTextFound;
 
-            nonTextFound = FALSE;  /* initial value */
-    
+            nonTextFound = false;  /* initial value */
+
             for (i = 0; i < bytesRead; ++i) {
                 char const c = buffer[i];
                 if (c < ' ' && !isspace(c) && c != ctlZ)
-                    nonTextFound = TRUE;
+                    nonTextFound = true;
             }
             retval = !nonTextFound;
         } else
-            retval = FALSE;
+            retval = false;
         FileClose(fileP);
     } else
-        retval = FALSE;
+        retval = false;
 
     return retval;
 }
 
 
- 
+
 static const char *
 mimeTypeGuessFromFile(MIMEType *   const MIMETypeP,
                       const char * const fileName) {
@@ -753,12 +766,12 @@ mimeTypeGuessFromFile(MIMEType *   const MIMETypeP,
 
     if (ext && MIMETypeP)
         retval = MIMETypeFromExt2(MIMETypeP, ext);
-    
+
     if (!retval) {
         if (fileContainsText(fileName))
             retval = "text/plain";
         else
-            retval = "application/octet-stream";  
+            retval = "application/octet-stream";
     }
     return retval;
 }
@@ -781,7 +794,7 @@ MIMETypeGuessFromFile(const char * const fileName) {
     return mimeTypeGuessFromFile(globalMimeTypeP, fileName);
 }
 
-                                  
+
 
 /******************************************************************************
 **
@@ -800,7 +813,7 @@ MIMETypeGuessFromFile(const char * const fileName) {
 **    documentation and/or other materials provided with the distribution.
 ** 3. The name of the author may not be used to endorse or promote products
 **    derived from this software without specific prior written permission.
-** 
+**
 ** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 ** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
