@@ -74,7 +74,7 @@
 #include "linklist.h"
 #include "girstring.h"
 
-#include "xmlrpc-c/util.h"
+#include "xmlrpc-c/util_int.h"
 #include "xmlrpc-c/string_int.h"
 #include "xmlrpc-c/select_int.h"
 #include "xmlrpc-c/client_int.h"
@@ -169,7 +169,7 @@ static void
 addMilliseconds(xmlrpc_timespec   const addend,
                 unsigned int      const adder,
                 xmlrpc_timespec * const sumP) {
-    
+
     unsigned int const million = 1000000;
     unsigned int const billion = 1000000000;
 
@@ -194,7 +194,7 @@ struct xmlrpc_client_transport {
            and consequently does not share things such as persistent
            connections and cookies with any other RPC.
         */
-    lock * syncCurlSessionLockP;
+    struct lock * syncCurlSessionLockP;
         /* Hold this lock while accessing or using *syncCurlSessionP.
            You're using the session from the time you set any
            attributes in it or start a transaction with it until any
@@ -211,7 +211,7 @@ struct xmlrpc_client_transport {
            the transaction with proper interruptibility.  The only Curl
            transaction ever attached to this multi manager is
            'syncCurlSessionP'.
-           
+
            This is constant (the handle, not the object).
         */
     curlMulti * asyncCurlMultiP;
@@ -220,7 +220,7 @@ struct xmlrpc_client_transport {
            interface.  Note that there may be multiple such Curl transactions
            simultaneously and one can't wait for a particular one to finish;
            the collection of asynchronous RPCs are an indivisible mass.
-           
+
            This is constant (the handle, not the object).
         */
     bool dontAdvertise;
@@ -234,7 +234,7 @@ struct xmlrpc_client_transport {
         */
     const char * userAgent;
         /* Information to include in a User-Agent HTTP header, reflecting
-           facilities outside of Xmlrpc-c.  
+           facilities outside of Xmlrpc-c.
 
            Null means none.
 
@@ -307,7 +307,7 @@ initWindowsStuff(xmlrpc_env * const envP ATTR_UNUSED) {
 
 #if MSVCRT
     /* This is CRITICAL so that cURL-Win32 works properly! */
-    
+
     /* So this commenter says, but I wonder why.  libcurl should do the
        required WSAStartup() itself, and it looks to me like it does.
        -Bryan 06.01.01
@@ -316,7 +316,7 @@ initWindowsStuff(xmlrpc_env * const envP ATTR_UNUSED) {
     WSADATA wsaData;
     int err;
     wVersionRequested = MAKEWORD(1, 1);
-    
+
     err = WSAStartup(wVersionRequested, &wsaData);
     if (err)
         xmlrpc_env_set_fault_formatted(
@@ -324,8 +324,8 @@ initWindowsStuff(xmlrpc_env * const envP ATTR_UNUSED) {
             "Winsock startup failed.  WSAStartup returned rc %d", err);
     else {
         if (LOBYTE(wsaData.wVersion) != 1 || HIBYTE(wsaData.wVersion) != 1) {
-            /* Tell the user that we couldn't find a useable */ 
-            /* winsock.dll. */ 
+            /* Tell the user that we couldn't find a useable */
+            /* winsock.dll. */
             xmlrpc_env_set_fault_formatted(
                 envP, XMLRPC_INTERNAL_ERROR, "Winsock reported that "
                 "it does not implement the requested version 1.1.");
@@ -381,6 +381,8 @@ pselectTimeout(xmlrpc_timeoutType const timeoutType,
     /* We assume there is work to do at least every 3 seconds, because
        the Curl multi manager often has retries and other scheduled work
        that doesn't involve file handles on which we can select().
+       One thing that might cause work to do without any file handle becoming
+       ready is Curl timing out a request.
     */
     switch (timeoutType) {
     case timeout_no:
@@ -400,14 +402,14 @@ pselectTimeout(xmlrpc_timeoutType const timeoutType,
     retval.tv_nsec = (uint32_t)((selectTimeoutMillisec % 1000) * million);
 
     return retval;
-}        
+}
 
 
 
 static void
 processCurlMessages(xmlrpc_env * const envP,
                     curlMulti *  const curlMultiP) {
-        
+
     bool endOfMessages;
 
     endOfMessages = false;   /* initial assumption */
@@ -455,7 +457,7 @@ waitForWork(xmlrpc_env *       const envP,
    checked, we will return immediately and if it arrives while we're waiting,
    we will return then.  Note that we can provide this service only because
    pselect() has the same atomic unblock/wait feature.
-   
+
    If sigmaskP is NULL, wait under whatever the current signal mask is.
 -----------------------------------------------------------------------------*/
     fd_set readFdSet;
@@ -472,6 +474,12 @@ waitForWork(xmlrpc_env *       const envP,
             /* There are no Curl file descriptors on which to wait.
                So either there's work to do right now or all transactions
                are already complete.
+
+               It may also be the case that there are Curl file descriptors on
+               which Caller should wait, but they are too high (> FD_SETSIZE)
+               to use with 'pselect'.  Libcurl doesn't provide a way to deal
+               with this case gracefully, so Caller will unfortunately end up
+               busywaiting for there to be work for libcurl to do.
             */
         } else {
             xmlrpc_timespec const pselectTimeoutArg =
@@ -485,7 +493,7 @@ waitForWork(xmlrpc_env *       const envP,
 
             rc = xmlrpc_pselect(maxFd+1, &readFdSet, &writeFdSet, &exceptFdSet,
                                 &pselectTimeoutArg, sigmaskP);
-            
+
             if (rc < 0 && errno != EINTR)
                 xmlrpc_faultf(envP, "Impossible failure of pselect() "
                               "with errno %d (%s)",
@@ -520,7 +528,7 @@ waitForWorkInt(xmlrpc_env *       const envP,
    waitForWork() will suffice.
 -----------------------------------------------------------------------------*/
     sigset_t callerBlockSet;
-#ifdef _WIN32
+#if MSVCRT
     waitForWork(envP, curlMultiP, timeoutType, deadline, &callerBlockSet);
 #else
     sigset_t allSignals;
@@ -530,7 +538,7 @@ waitForWorkInt(xmlrpc_env *       const envP,
     sigfillset(&allSignals);
 
     sigprocmask(SIG_BLOCK, &allSignals, &callerBlockSet);
-    
+
     if (*interruptP == 0)
         waitForWork(envP, curlMultiP, timeoutType, deadline, &callerBlockSet);
     else
@@ -635,13 +643,13 @@ finishCurlMulti(xmlrpc_env *       const envP,
     rpcStillRunning = true;  /* initial assumption */
     timedOut = false;
     curlCalledSinceInterrupt = false;
-    
+
     while (rpcStillRunning && !timedOut && !envP->fault_occurred) {
 
         if (interruptP && !curlCalledSinceInterrupt) {
             waitForWorkInt(envP, curlMultiP, timeoutType, deadline,
                            interruptP);
-        } else 
+        } else
             waitForWork(envP, curlMultiP, timeoutType, deadline, NULL);
 
         if (!envP->fault_occurred) {
@@ -658,7 +666,7 @@ finishCurlMulti(xmlrpc_env *       const envP,
             doCurlWork(envP, curlMultiP, &rpcStillRunning);
 
             xmlrpc_gettimeofday(&nowTime);
-            
+
             timedOut = (timeoutType == timeout_yes &&
                         timeIsAfter(nowTime, deadline));
         }
@@ -672,7 +680,7 @@ getTimeoutParm(xmlrpc_env *                          const envP,
                const struct xmlrpc_curl_xportparms * const curlXportParmsP,
                size_t                                const parmSize,
                unsigned int *                        const timeoutP) {
-               
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(timeout))
         *timeoutP = 0;
     else {
@@ -694,6 +702,37 @@ getTimeoutParm(xmlrpc_env *                          const envP,
 
 
 
+static void
+getConnectTimeoutParm(
+    xmlrpc_env *                          const envP,
+    const struct xmlrpc_curl_xportparms * const curlXportParmsP,
+    size_t                                const parmSize,
+    unsigned int *                        const timeoutP) {
+
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(connect_timeout))
+        *timeoutP = 0;
+    else {
+        if (curlHasNosignal()) {
+            /* libcurl represents the timeout in milliseconds with a 'long',
+               giving wrong results if it doesn't fit.
+            */
+            if ((unsigned)(long)(curlXportParmsP->connect_timeout) !=
+                curlXportParmsP->connect_timeout)
+                xmlrpc_faultf(envP, "Timeout value %u is too large.",
+                              curlXportParmsP->connect_timeout);
+            else
+                *timeoutP = curlXportParmsP->connect_timeout;
+        } else
+            xmlrpc_faultf(envP, "You cannot specify a "
+                          "'connect_timeout' parameter "
+                          "because the Curl library is too old and is not "
+                          "capable of doing timeouts except by using "
+                          "signals.  You need at least Curl 7.10");
+    }
+}
+
+
+
 static void
 setVerbose(bool * const verboseP) {
 
@@ -744,12 +783,12 @@ getXportParms(xmlrpc_env *                          const envP,
         transportP->userAgent = NULL;
     else
         transportP->userAgent = strdup(curlXportParmsP->user_agent);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(dont_advertise))
         transportP->dontAdvertise = false;
     else
         transportP->dontAdvertise = curlXportParmsP->dont_advertise;
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(network_interface))
         curlSetupP->networkInterface = NULL;
     else if (curlXportParmsP->network_interface == NULL)
@@ -762,8 +801,8 @@ getXportParms(xmlrpc_env *                          const envP,
         curlSetupP->sslVerifyPeer = true;
     else
         curlSetupP->sslVerifyPeer = !curlXportParmsP->no_ssl_verifypeer;
-        
-    if (!curlXportParmsP || 
+
+    if (!curlXportParmsP ||
         parmSize < XMLRPC_CXPSIZE(no_ssl_verifyhost))
         curlSetupP->sslVerifyHost = true;
     else
@@ -775,87 +814,87 @@ getXportParms(xmlrpc_env *                          const envP,
         curlSetupP->sslCert = NULL;
     else
         curlSetupP->sslCert = strdup(curlXportParmsP->ssl_cert);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(sslcerttype))
         curlSetupP->sslCertType = NULL;
     else if (curlXportParmsP->sslcerttype == NULL)
         curlSetupP->sslCertType = NULL;
     else
         curlSetupP->sslCertType = strdup(curlXportParmsP->sslcerttype);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(sslcertpasswd))
         curlSetupP->sslCertPasswd = NULL;
     else if (curlXportParmsP->sslcertpasswd == NULL)
         curlSetupP->sslCertPasswd = NULL;
     else
         curlSetupP->sslCertPasswd = strdup(curlXportParmsP->sslcertpasswd);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(sslkey))
         curlSetupP->sslKey = NULL;
     else if (curlXportParmsP->sslkey == NULL)
         curlSetupP->sslKey = NULL;
     else
         curlSetupP->sslKey = strdup(curlXportParmsP->sslkey);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(sslkeytype))
         curlSetupP->sslKeyType = NULL;
     else if (curlXportParmsP->sslkeytype == NULL)
         curlSetupP->sslKeyType = NULL;
     else
         curlSetupP->sslKeyType = strdup(curlXportParmsP->sslkeytype);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(sslkeypasswd))
         curlSetupP->sslKeyPasswd = NULL;
     else if (curlXportParmsP->sslkeypasswd == NULL)
         curlSetupP->sslKeyPasswd = NULL;
     else
         curlSetupP->sslKeyPasswd = strdup(curlXportParmsP->sslkeypasswd);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(sslengine))
         curlSetupP->sslEngine = NULL;
     else if (curlXportParmsP->sslengine == NULL)
         curlSetupP->sslEngine = NULL;
     else
         curlSetupP->sslEngine = strdup(curlXportParmsP->sslengine);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(sslengine_default))
         curlSetupP->sslEngineDefault = false;
     else
         curlSetupP->sslEngineDefault = !!curlXportParmsP->sslengine_default;
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(sslversion))
         curlSetupP->sslVersion = XMLRPC_SSLVERSION_DEFAULT;
     else
         curlSetupP->sslVersion = curlXportParmsP->sslversion;
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(cainfo))
         curlSetupP->caInfo = NULL;
     else if (curlXportParmsP->cainfo == NULL)
         curlSetupP->caInfo = NULL;
     else
         curlSetupP->caInfo = strdup(curlXportParmsP->cainfo);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(capath))
         curlSetupP->caPath = NULL;
     else if (curlXportParmsP->capath == NULL)
         curlSetupP->caPath = NULL;
     else
         curlSetupP->caPath = strdup(curlXportParmsP->capath);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(randomfile))
         curlSetupP->randomFile = NULL;
     else if (curlXportParmsP->randomfile == NULL)
         curlSetupP->randomFile = NULL;
     else
         curlSetupP->randomFile = strdup(curlXportParmsP->randomfile);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(egdsocket))
         curlSetupP->egdSocket = NULL;
     else if (curlXportParmsP->egdsocket == NULL)
         curlSetupP->egdSocket = NULL;
     else
         curlSetupP->egdSocket = strdup(curlXportParmsP->egdsocket);
-    
+
     if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(ssl_cipher_list))
         curlSetupP->sslCipherList = NULL;
     else if (curlXportParmsP->ssl_cipher_list == NULL)
@@ -905,6 +944,23 @@ getXportParms(xmlrpc_env *                          const envP,
         curlSetupP->referer = strdup(curlXportParmsP->referer);
 
     getTimeoutParm(envP, curlXportParmsP, parmSize, &curlSetupP->timeout);
+
+    getConnectTimeoutParm(envP, curlXportParmsP, parmSize,
+                          &curlSetupP->connectTimeout);
+
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(tcp_keepalive))
+        curlSetupP->tcpKeepalive = false;
+    else
+        curlSetupP->tcpKeepalive = curlXportParmsP->tcp_keepalive;
+
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(tcp_keepidle_sec))
+        curlSetupP->tcpKeepidle = 0;
+    else
+        curlSetupP->tcpKeepidle = curlXportParmsP->tcp_keepidle_sec;
+    if (!curlXportParmsP || parmSize < XMLRPC_CXPSIZE(tcp_keepintvl_sec))
+        curlSetupP->tcpKeepintvl = 0;
+    else
+        curlSetupP->tcpKeepintvl = curlXportParmsP->tcp_keepintvl_sec;
 }
 
 
@@ -1018,7 +1074,7 @@ makeSyncCurlSession(xmlrpc_env *                     const envP,
         if (!envP->fault_occurred) {
             /* We'll need a multi manager to actually execute this session: */
             transportP->syncCurlMultiP = curlMulti_create();
-        
+
             if (transportP->syncCurlMultiP == NULL)
                 xmlrpc_faultf(envP, "Unable to create Curl multi manager for "
                               "synchronous RPCs");
@@ -1028,7 +1084,7 @@ makeSyncCurlSession(xmlrpc_env *                     const envP,
         }
         if (envP->fault_occurred)
             transportP->syncCurlSessionLockP->destroy(
-                transportP->syncCurlSessionLockP); 
+                transportP->syncCurlSessionLockP);
     }
 }
 
@@ -1042,12 +1098,12 @@ unmakeSyncCurlSession(struct xmlrpc_client_transport * const transportP) {
     destroySyncCurlSession(transportP->syncCurlSessionP);
 
     transportP->syncCurlSessionLockP->destroy(
-        transportP->syncCurlSessionLockP); 
+        transportP->syncCurlSessionLockP);
 }
 
 
 
-static void 
+static void
 create(xmlrpc_env *                      const envP,
        int                               const flags ATTR_UNUSED,
        const char *                      const appname ATTR_UNUSED,
@@ -1058,7 +1114,7 @@ create(xmlrpc_env *                      const envP,
 /*----------------------------------------------------------------------------
    This does the 'create' operation for a Curl client transport.
 -----------------------------------------------------------------------------*/
-    const struct xmlrpc_curl_xportparms * const curlXportParmsP = 
+    const struct xmlrpc_curl_xportparms * const curlXportParmsP =
         transportparmsP;
 
     struct xmlrpc_client_transport * transportP;
@@ -1072,16 +1128,16 @@ create(xmlrpc_env *                      const envP,
         transportP->interruptP = NULL;
 
         transportP->asyncCurlMultiP = curlMulti_create();
-        
+
         if (transportP->asyncCurlMultiP == NULL)
             xmlrpc_faultf(envP, "Unable to create Curl multi manager for "
                           "asynchronous RPCs");
         else {
             getXportParms(envP, curlXportParmsP, parm_size, transportP);
-            
+
             if (!envP->fault_occurred) {
                 makeSyncCurlSession(envP, transportP);
-                
+
                 if (envP->fault_occurred)
                     freeXportParms(transportP);
             }
@@ -1111,11 +1167,11 @@ assertNoOutstandingCurlWork(curlMulti * const curlMultiP) {
     xmlrpc_env env;
     bool immediateWorkToDo;
     int runningHandles;
-    
+
     xmlrpc_env_init(&env);
-    
+
     curlMulti_perform(&env, curlMultiP, &immediateWorkToDo, &runningHandles);
-    
+
     /* We know the above was a no-op, since we're asserting that there
        is no outstanding work.
     */
@@ -1127,7 +1183,7 @@ assertNoOutstandingCurlWork(curlMulti * const curlMultiP) {
 
 
 
-static void 
+static void
 destroy(struct xmlrpc_client_transport * const clientTransportP) {
 /*----------------------------------------------------------------------------
    This does the 'destroy' operation for a Curl client transport.
@@ -1229,7 +1285,7 @@ createRpc(xmlrpc_env *                     const envP,
           const xmlrpc_server_info *       const serverP,
           xmlrpc_mem_block *               const callXmlP,
           xmlrpc_mem_block *               const responseXmlP,
-          xmlrpc_transport_asynch_complete       complete, 
+          xmlrpc_transport_asynch_complete       complete,
           xmlrpc_transport_progress              progress,
           struct xmlrpc_call_info *        const callInfoP,
           rpc **                           const rpcPP) {
@@ -1260,7 +1316,7 @@ createRpc(xmlrpc_env *                     const envP,
         curlTransaction_create(envP,
                                curlSessionP,
                                serverP,
-                               callXmlP, responseXmlP, 
+                               callXmlP, responseXmlP,
                                clientTransportP->dontAdvertise,
                                clientTransportP->userAgent,
                                &clientTransportP->curlSetupStuff,
@@ -1280,7 +1336,7 @@ createRpc(xmlrpc_env *                     const envP,
 
 
 
-static void 
+static void
 destroyRpc(rpc * const rpcP) {
 
     XMLRPC_ASSERT_PTR_OK(rpcP);
@@ -1361,7 +1417,7 @@ curlTransactionProgress(void * const context,
 
    The curlTransaction calls this once a second telling us how much
    data has transferred.  If the transport user has set up a progress
-   function, we call that with this progress information.  That 
+   function, we call that with this progress information.  That
    function might e.g. display a progress bar.
 
    Additionally, the curlTransaction gives us the opportunity to tell it
@@ -1400,8 +1456,8 @@ curlTransactionProgress(void * const context,
 
 
 
-static void 
-sendRequest(xmlrpc_env *                     const envP, 
+static void
+sendRequest(xmlrpc_env *                     const envP,
             struct xmlrpc_client_transport * const clientTransportP,
             const xmlrpc_server_info *       const serverP,
             xmlrpc_mem_block *               const callXmlP,
@@ -1423,7 +1479,7 @@ sendRequest(xmlrpc_env *                     const envP,
     responseXmlP = XMLRPC_MEMBLOCK_NEW(char, envP, 0);
     if (!envP->fault_occurred) {
         CURL * const curlSessionP = curl_easy_init();
-    
+
         if (curlSessionP == NULL)
             xmlrpc_faultf(envP, "Could not create Curl session.  "
                           "curl_easy_init() failed.");
@@ -1431,10 +1487,10 @@ sendRequest(xmlrpc_env *                     const envP,
             createRpc(envP, clientTransportP, curlSessionP, serverP,
                       callXmlP, responseXmlP, complete, progress, callInfoP,
                       &rpcP);
-            
+
             if (!envP->fault_occurred) {
                 startRpc(envP, rpcP);
-                
+
                 if (envP->fault_occurred)
                     destroyRpc(rpcP);
             }
@@ -1453,7 +1509,7 @@ sendRequest(xmlrpc_env *                     const envP,
 
 
 
-static void 
+static void
 finishAsynch(
     struct xmlrpc_client_transport * const clientTransportP,
     xmlrpc_timeoutType               const timeoutType,
@@ -1477,7 +1533,7 @@ finishAsynch(
    ready to finish at that moment.  The implementation would be little
    more than wrapping curl_multi_fdset() and curl_multi_perform().
 
-   Note that the user can call this multiple times, due to timeouts,
+   Note that the user can call this multiple times, because of timeouts,
    but must eventually call it once with no timeout so he
    knows that all the RPCs are finished.  Either that or terminate the
    process so it doesn't matter if RPCs are still going.
@@ -1488,7 +1544,7 @@ finishAsynch(
         /* The datetime after which we should quit waiting */
 
     xmlrpc_env_init(&env);
-    
+
     if (timeoutType == timeout_yes) {
         xmlrpc_timespec waitStartTime;
         xmlrpc_gettimeofday(&waitStartTime);
@@ -1509,7 +1565,7 @@ finishAsynch(
 
        Note that a failure of finish_curlMulti() does not mean that
        a session completed with an error or an RPC completed with an
-       error.  Those things are reported up through the user's 
+       error.  Those things are reported up through the user's
        xmlrpc_transport_asynch_complete routine.  A failure here is
        something that stopped us from calling that.
 
@@ -1583,7 +1639,7 @@ setupGlobalConstants(xmlrpc_env * const envP) {
         CURLcode rc;
 
         rc = curl_global_init(CURL_GLOBAL_ALL);
-        
+
         if (rc != CURLE_OK)
             xmlrpc_faultf(envP, "curl_global_init() failed with code %d", rc);
     }
