@@ -3,17 +3,15 @@ Copyright (c) 1998, 1999 Thai Open Source Software Center Ltd
 See the file copying.txt for copying permission.
 */
 
-#ifndef XmlTok_INCLUDED
-#define XmlTok_INCLUDED 1
+#ifndef XMLTOK_H_INCLUDED
+#define XMLTOK_H_INCLUDED
+
+#include "xmlrpc-c/c_util.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifndef XMLTOKAPI
-#define XMLTOKAPI /* as nothing */
-#endif
-
 /* The following token may be returned by XmlContentTok */
 #define XML_TOK_TRAILING_RSQB -5 /* ] or ]] at the end of the scan; might be start of
                                     illegal ]]> sequence */
@@ -117,6 +115,39 @@ typedef struct {
 struct encoding;
 typedef struct encoding ENCODING;
 
+/* It's pretty silly to have the 'const' on the argument names below, but MSVC
+   thinks the definitions of Utf8Converter functions that do have that
+   qualifier on the argument variables don't match this declaration if we
+   don't.
+*/
+typedef void Utf8Converter(const ENCODING * const encP,
+                           const char **    const fromP,
+                           const char *     const fromLim,
+                           char **          const toP,
+                           const char *     const toLim);
+/*----------------------------------------------------------------------------
+   A Utf8Converter is an encoder method that converts from the encoder's
+   code to UTF-8:
+
+   Convert the string that starts at *fromP and ends at 'fromLim' to UTF8 in
+   the buffer that starts at *toP and ends at 'toLim'.
+
+   Go from left to right and stop when the output buffer is full.
+
+   Note that the buffer can be full while still having a few bytes left in it
+   because an input character may require multiple bytes of the output buffer.
+
+   The encoding may require up to 4 UTF-8 bytes for a single input character.
+
+   Leave *fromP and *toP pointing after the last character converted.
+
+   The output buffer must be at least 4 bytes.  Results are undefined if not.
+
+   We necessarily advance *fromP at least one character if 'fromLim' allows.
+-----------------------------------------------------------------------------*/
+
+
+    
 struct encoding {
   int (*scanners[XML_N_STATES])(const ENCODING *,
 			        const char *,
@@ -130,7 +161,7 @@ struct encoding {
 	          const char *, const char *);
   int (*nameMatchesAscii)(const ENCODING *,
 			  const char *, const char *, const char *);
-  int (*nameLength)(const ENCODING *, const char *);
+  size_t (*nameLength)(const ENCODING *, const char *);
   const char *(*skipS)(const ENCODING *, const char *);
   int (*getAtts)(const ENCODING *enc, const char *ptr,
 	         int attsMax, ATTRIBUTE *atts);
@@ -142,11 +173,7 @@ struct encoding {
 			 POSITION *);
   int (*isPublicId)(const ENCODING *enc, const char *ptr, const char *end,
 		    const char **badPtr);
-  void (*utf8Convert)(const ENCODING *enc,
-		      const char **fromP,
-		      const char *fromLim,
-		      char **toP,
-		      const char *toLim);
+  Utf8Converter * utf8Convert;
   void (*utf16Convert)(const ENCODING *enc,
 		       const char **fromP,
 		       const char *fromLim,
@@ -242,7 +269,8 @@ typedef struct {
   const ENCODING **encPtr;
 } INIT_ENCODING;
 
-XMLTOKAPI int
+XMLRPC_DLLEXPORT
+int
 xmlrpc_XmlParseXmlDecl(int               const isGeneralTextEntity,
                        const ENCODING *  const enc,
                        const char *      const ptr,
@@ -253,34 +281,42 @@ xmlrpc_XmlParseXmlDecl(int               const isGeneralTextEntity,
                        const ENCODING ** const namedEncodingPtr,
                        int *             const standalonePtr);
 
-XMLTOKAPI const ENCODING *
+XMLRPC_DLLEXPORT
+const ENCODING *
 xmlrpc_XmlGetUtf8InternalEncoding(void);
 
-XMLTOKAPI const ENCODING *
+XMLRPC_DLLEXPORT
+const ENCODING *
 xmlrpc_XmlGetUtf16InternalEncoding(void);
 
-XMLTOKAPI int
+XMLRPC_DLLEXPORT
+int
 xmlrpc_XmlInitEncoding(INIT_ENCODING *   const p,
                        const ENCODING ** const encPtr,
                        const char *      const name);
 
-XMLTOKAPI int
+XMLRPC_DLLEXPORT
+int
 xmlrpc_XmlUtf8Encode(int    const c,
                      char * const buf);
 
-XMLTOKAPI int
+XMLRPC_DLLEXPORT
+int
 xmlrpc_XmlUtf16Encode(int              const charNum,
                       unsigned short * const buf);
 
-XMLTOKAPI int
+XMLRPC_DLLEXPORT
+int
 xmlrpc_XmlSizeOfUnknownEncoding(void);
 
-XMLTOKAPI ENCODING *
+XMLRPC_DLLEXPORT
+ENCODING *
 xmlrpc_XmlInitUnknownEncoding(void * const mem,
                               int *  const table,
                               int (*convert)(void *userData, const char *p),
                               void * const userData);
 
+XMLRPC_DLLEXPORT
 int
 xmlrpc_XmlParseXmlDeclNS(int               const isGeneralTextEntity,
                          const ENCODING *  const enc,
@@ -292,17 +328,21 @@ xmlrpc_XmlParseXmlDeclNS(int               const isGeneralTextEntity,
                          const ENCODING ** const namedEncodingPtr,
                          int *             const standalonePtr);
 
+XMLRPC_DLLEXPORT
 int
 xmlrpc_XmlInitEncodingNS(INIT_ENCODING *   const p,
                          const ENCODING ** const encPtr,
                          const char *      const name);
 
+XMLRPC_DLLEXPORT
 const ENCODING *
 xmlrpc_XmlGetUtf8InternalEncodingNS(void);
 
+XMLRPC_DLLEXPORT
 const ENCODING *
 xmlrpc_XmlGetUtf16InternalEncodingNS(void);
 
+XMLRPC_DLLEXPORT
 ENCODING *
 xmlrpc_XmlInitUnknownEncodingNS(void * const mem,
                                 int *  const table,
