@@ -145,9 +145,9 @@ currentDocumentPosition(Tokenizer * const tokP) {
 
 
 static void
-setParseErr(xmlrpc_env *   const envP,
-            Tokenizer * const tokP,
-            const char *   const format,
+setParseErr(xmlrpc_env * const envP,
+            Tokenizer *  const tokP,
+            const char * const format,
             ...) {
 
     struct docPosition const pos = currentDocumentPosition(tokP);
@@ -175,8 +175,8 @@ setParseErr(xmlrpc_env *   const envP,
 
 
 static void
-finishStringToken(xmlrpc_env *   const envP,
-                  Tokenizer * const tokP) {
+finishStringToken(xmlrpc_env * const envP,
+                  Tokenizer *  const tokP) {
 
     ++tokP->end;
 
@@ -203,7 +203,7 @@ finishStringToken(xmlrpc_env *   const envP,
 
                 while (isxdigit(*cur) && cur - tokP->end < 4)
                     ++cur;
-                
+
                 if (cur - tokP->end < 4)
                     setParseErr(envP, tokP,
                                 "hex unicode must contain 4 digits.  "
@@ -211,7 +211,10 @@ finishStringToken(xmlrpc_env *   const envP,
                 else
                     tokP->end = cur;
             } break;
-
+            case '\0':
+                setParseErr(envP, tokP, "JSON document ends in the middle "
+                            "of a backslash escape sequence");
+                break;
             default:
                 setParseErr(envP, tokP, "unknown escape character "
                             "after backslash: '%c'", *tokP->end);
@@ -222,7 +225,7 @@ finishStringToken(xmlrpc_env *   const envP,
     if (!envP->fault_occurred) {
         if (*tokP->end == '\0')
             setParseErr(envP, tokP, "JSON document ends in the middle "
-                        "of a backslash escape sequence");
+                        "of a string literal");
         else {
             ++tokP->end;
             tokP->size = (tokP->end - tokP->begin) - 1;
@@ -265,7 +268,7 @@ isFloat(const char * const token,
    point number.
 
    N.B. This is true of any integer.
-   
+
    We don't accept plus signs.
 
    Examples of valid floating point:  0, 32, 32.5, , 32.500,
@@ -382,7 +385,7 @@ advanceToNextToken(Tokenizer * const tokP) {
 
     while (*tokP->begin != '\0' &&
            (isspace(*tokP->begin) || atComment(tokP))) {
-        
+
         advancePastWhiteSpace(tokP);
 
         advancePastComments(tokP);
@@ -392,8 +395,8 @@ advanceToNextToken(Tokenizer * const tokP) {
 
 
 static void
-getToken(xmlrpc_env *   const envP,
-         Tokenizer * const tokP) {
+getToken(xmlrpc_env * const envP,
+         Tokenizer *  const tokP) {
 
     /* The token starts where the last one left off */
     tokP->begin = tokP->end;
@@ -407,7 +410,7 @@ getToken(xmlrpc_env *   const envP,
         tokP->size = tokP->end - tokP->begin;
     } else {
         tokP->end = tokP->begin;  /* initial value */
-    
+
         if (*tokP->begin == '{') {
             finishDelimiterToken(tokP);
             tokP->type = typeOpenBrace;
@@ -547,7 +550,7 @@ getBackslashSequence(xmlrpc_env *       const envP,
         buffer[0] = '\t';
         tsize = 1;
         *nBytesConsumedP = 1;
-        break;    
+        break;
     case 'u': {
         long digit;
         strncpy(buffer, cur + 1, 4);
@@ -569,12 +572,14 @@ getBackslashSequence(xmlrpc_env *       const envP,
 
 
 static void
-unescapeString(xmlrpc_env *       const envP,
-               const char *       const begin,
-               const char *       const end,
-               xmlrpc_mem_block * const memBlockP) {
+unescapeString(xmlrpc_env *        const envP,
+               const char *        const begin,
+               const char *        const end,
+               xmlrpc_mem_block ** const memBlockPP) {
 
-    XMLRPC_MEMBLOCK_INIT(char, envP, memBlockP, 0);
+    xmlrpc_mem_block * memBlockP;
+
+    memBlockP = XMLRPC_MEMBLOCK_NEW(char, envP, 0);
 
     if (!envP->fault_occurred) {
         const char * cur;
@@ -582,7 +587,7 @@ unescapeString(xmlrpc_env *       const envP,
 
         cur = begin;
         last = cur;
-            
+
         while (cur != end && !envP->fault_occurred) {
             if (*cur == '\\') {
                 if (cur != last) {
@@ -618,8 +623,9 @@ unescapeString(xmlrpc_env *       const envP,
             XMLRPC_MEMBLOCK_APPEND(char, envP, memBlockP, "", 1);
         }
         if (envP->fault_occurred)
-            XMLRPC_MEMBLOCK_CLEAN(char, memBlockP);
+            XMLRPC_MEMBLOCK_FREE(char, memBlockP);
     }
+    *memBlockPP = memBlockP;
 }
 
 
@@ -628,7 +634,7 @@ static xmlrpc_value *
 makeUtf8String(xmlrpc_env * const envP,
                const char * const begin,
                const char * const end) {
-/*---------------------------------------------------------------------------- 
+/*----------------------------------------------------------------------------
     Copy a json string directly into a string value, and convert any json
     escaping (\uXXXX) to something acceptable to the internal string handling.
 
@@ -643,7 +649,7 @@ makeUtf8String(xmlrpc_env * const envP,
         valP->_wcs_block = NULL;
 
         if (!envP->fault_occurred)
-            unescapeString(envP, begin, end, &valP->_block);
+            unescapeString(envP, begin, end, &valP->blockP);
 
         if (envP->fault_occurred)
             xmlrpc_DECREF(valP);
@@ -665,7 +671,7 @@ stringTokenValue(xmlrpc_env * const envP,
     assert(tokP->end >= tokP->begin + 2);
     assert(*tokP->begin == '"');
     assert(*(tokP->end-1) == '"');
-    
+
     valP = makeUtf8String(&env, tokP->begin + 1, tokP->end - 1);
 
     if (env.fault_occurred) {
@@ -700,7 +706,7 @@ integerTokenValue(xmlrpc_env * const envP,
                     tokP->begin, env.fault_string);
     else
         valP = xmlrpc_i8_new(envP, value);
-    
+
     xmlrpc_env_clean(&env);
 
     return valP;
@@ -766,7 +772,7 @@ parseList(xmlrpc_env *   const envP,
     XMLRPC_ASSERT_ENV_OK(envP);
 
     retval = xmlrpc_array_new(envP);
-    
+
     if (!envP->fault_occurred) {
         bool endOfList;
         for (endOfList = false; !endOfList && !envP->fault_occurred; ) {
@@ -823,13 +829,13 @@ parseObjectMember(xmlrpc_env *   const envP,
     xmlrpc_env_init(&env);
 
     /* The current token is the string which is the member name: */
-    assert(tokP->type = typeString);
+    assert(tokP->type == typeString);
     assert(tokP->end >= tokP->begin + 2);
     assert(*tokP->begin == '"');
     assert(*(tokP->end-1) == '"');
-                      
+
     keyP = makeUtf8String(&env, tokP->begin + 1, tokP->end - 1);
-                
+
     if (env.fault_occurred)
         setParseErr(envP, tokP, "Error in what is supposed to be "
                     "the key of a member of an object: %s",
@@ -852,7 +858,7 @@ parseObjectMember(xmlrpc_env *   const envP,
 }
 
 
- 
+
 static xmlrpc_value *
 parseObject(xmlrpc_env *   const envP,
             Tokenizer * const tokP) {
@@ -878,7 +884,7 @@ parseObject(xmlrpc_env *   const envP,
 
                     if (!envP->fault_occurred) {
                         getToken(envP, tokP);
-                        
+
                         if (!envP->fault_occurred) {
                             if (tokP->type == typeComma) {
                                 /* member separator; keep going */
@@ -918,9 +924,9 @@ parseValue(xmlrpc_env * const envP,
            Tokenizer *  const tokP) {
 
     xmlrpc_value * retval;
-    
+
     XMLRPC_ASSERT_ENV_OK(envP);
-        
+
     switch (tokP->type) {
 
     case typeOpenBracket:
@@ -930,7 +936,7 @@ parseValue(xmlrpc_env * const envP,
     case typeOpenBrace:
         retval = parseObject(envP, tokP);
         break;
-            
+
     case typeNull:
         retval = xmlrpc_nil_new(envP);
         break;
@@ -950,7 +956,7 @@ parseValue(xmlrpc_env * const envP,
     case typeInteger:
         retval = integerTokenValue(envP, tokP);
         break;
-        
+
     case typeFloat:
         retval = xmlrpc_double_new(envP, strtod(tokP->begin, NULL));
         break;
@@ -958,7 +964,7 @@ parseValue(xmlrpc_env * const envP,
     case typeString:
         retval = stringTokenValue(envP, tokP);
         break;
-            
+
     default:
         retval = NULL;
         setParseErr(envP, tokP, "Invalid token "
@@ -978,9 +984,9 @@ xmlrpc_parse_json(xmlrpc_env * const envP,
 
     xmlrpc_value * retval = retval;
     Tokenizer tok;
-    
+
     XMLRPC_ASSERT_ENV_OK(envP);
-    
+
     initializeTokenizer(&tok, str);
 
     getToken(envP, &tok);
@@ -1084,10 +1090,10 @@ appendEscapeSeq(xmlrpc_env *       const envP,
    This is e.g. "\t" for tab, or "\u001C" for something exotic.
 -----------------------------------------------------------------------------*/
     unsigned int size;
-    char buffer[6];
+    char buffer[6+1];
     char slashChar;
         /* Character that goes after the backslash, including 'u' for \uHHHH */
-    
+
     switch (c) {
     case '"' : slashChar = '"';  break; /* U+0022 */
     case '\\': slashChar = '\\'; break; /* U+005C */
@@ -1102,7 +1108,7 @@ appendEscapeSeq(xmlrpc_env *       const envP,
 
     buffer[0] = '\\';
     buffer[1] = slashChar;
-    
+
     if (slashChar == 'u') {
         sprintf(&buffer[2], "%04x", c);
         size = 6;  /* \u1234 */
@@ -1129,7 +1135,7 @@ makeJsonString(xmlrpc_env *       const envP,
     const char * last;
 
     last = cur = begin;
-    
+
     while (cur != end && !envP->fault_occurred) {
         unsigned char const c = *cur;
 
@@ -1139,7 +1145,7 @@ makeJsonString(xmlrpc_env *       const envP,
                the characters before it to the output.
             */
             XMLRPC_MEMBLOCK_APPEND(char, envP, outP, last, cur - last);
-            
+
             if (!envP->fault_occurred) {
                 appendEscapeSeq(envP, outP, c);
 
@@ -1153,9 +1159,6 @@ makeJsonString(xmlrpc_env *       const envP,
     /* Copy all characters since the last escaped character to the output */
     if (cur != last)
         XMLRPC_MEMBLOCK_APPEND(char, envP, outP, last, cur - last);
-
-    if (envP->fault_occurred)
-        XMLRPC_MEMBLOCK_CLEAN(char, outP);
 }
 
 
@@ -1202,7 +1205,7 @@ serializeI8(xmlrpc_env *       const envP,
     xmlrpc_int64 value;
 
     xmlrpc_read_i8(envP, valP, &value);
-            
+
     formatOut(envP, outP, "%" XMLRPC_PRId64, value);
 }
 
@@ -1212,7 +1215,7 @@ static void
 serializeBool(xmlrpc_env *       const envP,
               xmlrpc_value *     const valP,
               xmlrpc_mem_block * const outP) {
-    
+
     xmlrpc_bool value;
     xmlrpc_read_bool(envP, valP, &value);
 
@@ -1225,7 +1228,7 @@ static void
 serializeDouble(xmlrpc_env *       const envP,
                 xmlrpc_value *     const valP,
                 xmlrpc_mem_block * const outP) {
-    
+
     xmlrpc_double value;
     xmlrpc_read_double(envP, valP, &value);
 
@@ -1251,16 +1254,16 @@ serializeDatetime(xmlrpc_env *       const envP,
 }
 
 
-        
+
 static void
 serializeString(xmlrpc_env * const envP,
                 xmlrpc_value *     const valP,
                 xmlrpc_mem_block * const outP) {
-    
+
     formatOut(envP, outP, "\"");
-    
+
     makeJsonStringFromXmlRpc(envP, valP, outP);
-    
+
     formatOut(envP, outP, "\"");
 }
 
@@ -1286,12 +1289,12 @@ serializeBitstring(xmlrpc_env *       const envP,
         if (!envP->fault_occurred) {
 
             formatOut(envP, outP, "\"");
-            
+
             XMLRPC_MEMBLOCK_APPEND(
                 char, envP, outP,
                 XMLRPC_MEMBLOCK_CONTENTS(char, base64P),
                 XMLRPC_MEMBLOCK_SIZE(char, base64P));
-            
+
             if (!envP->fault_occurred)
                 formatOut(envP, outP, "\"");
 
@@ -1319,7 +1322,7 @@ serializeArray(xmlrpc_env *       const envP,
         for (i = 0; i < size && !envP->fault_occurred; ++i) {
             xmlrpc_value * const itemP =
                 xmlrpc_array_get_item(envP, valP, i);
-                    
+
             if (!envP->fault_occurred) {
                 if (!envP->fault_occurred) {
                     serializeValue(envP, itemP, level + 1, outP);
@@ -1337,7 +1340,7 @@ serializeArray(xmlrpc_env *       const envP,
             }
         }
     }
-} 
+}
 
 
 
@@ -1349,17 +1352,17 @@ serializeStructMember(xmlrpc_env *       const envP,
                       xmlrpc_mem_block * const outP) {
 
     serializeValue(envP, memberKeyP, level, outP);
-                    
+
     if (!envP->fault_occurred) {
         formatOut(envP, outP, ":");
-        
+
         if (!envP->fault_occurred)
             serializeValue(envP, memberValueP, level, outP);
     }
 }
 
 
- 
+
 static void
 serializeStruct(xmlrpc_env *       const envP,
                 xmlrpc_value *     const valP,
@@ -1370,7 +1373,7 @@ serializeStruct(xmlrpc_env *       const envP,
         formatOut(envP, outP, "{\n");
         if (!envP->fault_occurred) {
             unsigned int const size = xmlrpc_struct_size(envP, valP);
-            
+
             if (!envP->fault_occurred) {
                 unsigned int i;
                 for (i = 0; i < size && !envP->fault_occurred; ++i) {
@@ -1383,7 +1386,7 @@ serializeStruct(xmlrpc_env *       const envP,
                     if (!envP->fault_occurred) {
                         serializeStructMember(envP, memberKeyP, memberValueP,
                                               level + 1, outP);
-                        
+
                         if (!envP->fault_occurred && i < size - 1)
                             XMLRPC_MEMBLOCK_APPEND(char, envP, outP, ",\n", 2);
                     }
@@ -1405,11 +1408,15 @@ serializeValue(xmlrpc_env *       const envP,
                xmlrpc_value *     const valP,
                unsigned int       const level,
                xmlrpc_mem_block * const outP) {
+/*----------------------------------------------------------------------------
+   Generate JSON to represent the value *valP.  Append it to *outP.
 
+   The JSON consists of lines of text.  Indent them 'level' levels.
+-----------------------------------------------------------------------------*/
     XMLRPC_ASSERT_ENV_OK(envP);
 
     indent(envP, level, outP);
-        
+
     switch (xmlrpc_value_type(valP)) {
     case XMLRPC_TYPE_INT:
         serializeInt(envP, valP, outP);
@@ -1437,7 +1444,7 @@ serializeValue(xmlrpc_env *       const envP,
 
     case XMLRPC_TYPE_BASE64:
         serializeBitstring(envP, valP, outP);
-        break;      
+        break;
 
     case XMLRPC_TYPE_ARRAY:
         serializeArray(envP, valP, level, outP);
@@ -1471,6 +1478,8 @@ void
 xmlrpc_serialize_json(xmlrpc_env *       const envP,
                       xmlrpc_value *     const valP,
                       xmlrpc_mem_block * const outP) {
-
+/*----------------------------------------------------------------------------
+   Generate JSON to represent the value *valP.  Append it to *outP.
+-----------------------------------------------------------------------------*/
     serializeValue(envP, valP, 0, outP);
 }
