@@ -208,7 +208,11 @@ xmlrpc_server_cgi_process_call(xmlrpc_registry * const registryP) {
         code = 405; message = "Method Not Allowed";
         XMLRPC_FAIL(&env, XMLRPC_INTERNAL_ERROR, "Expected HTTP method POST");
     }
-    if (!type || !xmlrpc_strneq(type, "text/xml", strlen("text/xml"))) {
+    if (!type) {
+        code = 400; message = "Bad Request";
+        XMLRPC_FAIL(&env, XMLRPC_INTERNAL_ERROR,
+                    "No content type header field");
+    } else if (!xmlrpc_strneq(type, "text/xml", strlen("text/xml"))) {
 	char *template = "Expected content type: \"text/xml\", received: \"%s\"";
 	size_t err_len = strlen(template) + strlen(type) + 1;
 	char *err = malloc(err_len);
