@@ -15,12 +15,15 @@
 #include "xmlrpc-c/girerr.hpp"
 using girerr::throwf;
 
-#include <features.h>  // for __BEGIN_DECLS
+#ifdef __cplusplus
+extern "C" {
+#endif
 
-__BEGIN_DECLS
 #include "dumpvalue.h"  /* An internal Xmlrpc-c header file ! */
-__END_DECLS
 
+#ifdef __cplusplus
+}
+#endif
 
 #include <xmlrpc-c/base.hpp>
 #include <xmlrpc-c/client.hpp>
